///////////////////////////////////////////////////////////////////////////////
// CPinnableDialog.cpp : Implementation of the CPinnableDialog class.
//
// Copyright 1999 David Hill - dhill@PrincipiaMathematica.com.
//
///////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "PinnableDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//-----------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CPinnableDialog, CDialog)
	//{{AFX_MSG_MAP(CPinnableDialog)
	ON_WM_CLOSE()
	ON_MESSAGE( WM_ACTIVATE, OnActivate )
	ON_WM_SYSCOMMAND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//-----------------------------------------------------------------------------

CPinnableDialog::CPinnableDialog( UINT  nDlgID,
							      UINT  nPinnedIconID,
							      UINT  nUnpinnedIconID,
							      CWnd* pParent )
: CDialog( nDlgID, pParent )
{
	//{{AFX_DATA_INIT(CPinnableDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_hPinned             = AfxGetApp()->LoadIcon(nPinnedIconID);
	m_hUnpinned           = AfxGetApp()->LoadIcon(nUnpinnedIconID);
    m_nTemplateID         = nDlgID;
	m_bPinned             = FALSE;
	m_bDeactivateAsCancel = TRUE;
	ASSERT( m_hPinned );
	ASSERT( m_hUnpinned );
}

//-----------------------------------------------------------------------------

void CPinnableDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPinnableDialog)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

//-----------------------------------------------------------------------------

BOOL CPinnableDialog::Show( BOOL bPinned )
{
	BOOL bRet = TRUE;

	// Show the dialog.
	if ( GetSafeHwnd() == 0 )
	{
		bRet = CDialog::Create( m_nTemplateID );
	}

	// Set the pinned state icon.
	if ( bRet )
	{
		m_bPinned = bPinned;
		UpdateIcon();
	}

	return bRet;
}

//-----------------------------------------------------------------------------

BOOL CPinnableDialog::Hide()
{
	BOOL bRet = TRUE;

	if ( GetSafeHwnd() != 0 )
	{
		bRet = DestroyWindow();
	}

	return bRet;
}

//-----------------------------------------------------------------------------

void CPinnableDialog::OnClose() 
{	
	if ( GetParent() != NULL )
	{
		// Inform the parent.
		GetParent()->PostMessage( WM_PINUPCLOSE, IDCANCEL, 0 );
	}
	else
	{
		// Standard modal behaviour.
		CDialog::OnCancel();
	}
}

//-----------------------------------------------------------------------------

void CPinnableDialog::OnCancel() 
{
	if ( GetParent() != NULL )
	{
		// Inform the parent.
		GetParent()->PostMessage( WM_PINUPCLOSE, IDCANCEL, 0 );
	}
	else
	{
		// Standard modal behaviour.
		CDialog::OnCancel();
	}
}

//-----------------------------------------------------------------------------

void CPinnableDialog::OnOK() 
{	
	if ( GetParent() != NULL )
	{
		// Inform the parent.
		UpdateData(TRUE);
		GetParent()->PostMessage( WM_PINUPCLOSE, IDOK, 0 );
	}
	else
	{
		// Standard modal behaviour.
		CDialog::OnOK();
	}
}

//-----------------------------------------------------------------------------

LRESULT CPinnableDialog::OnActivate( WPARAM wParam, LPARAM lParam )
{
	lParam;		// Unused.
	WORD fActive = LOWORD(wParam);

	if ( !m_bPinned && fActive == WA_INACTIVE && GetParent() != NULL )
	{
		// Inform the parent.
		UpdateData(TRUE);
		GetParent()->PostMessage( WM_PINUPCLOSE, m_bDeactivateAsCancel ? IDCANCEL : IDOK, 0 );
	}
	return 0L;
}

//-----------------------------------------------------------------------------

BOOL CPinnableDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

	UpdateIcon();

	return TRUE;  // Return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

//-----------------------------------------------------------------------------

void CPinnableDialog::UpdateIcon()
{
	if ( m_bPinned )
	{
		SetIcon(m_hPinned, TRUE);
		SetIcon(m_hPinned, FALSE);
	} else {
		SetIcon(m_hUnpinned, TRUE);
		SetIcon(m_hUnpinned, FALSE);
	}
}

//-----------------------------------------------------------------------------

void CPinnableDialog::OnSysCommand( UINT nID, LPARAM lParam )
{
	// Prevent the system menu from being shown and the dialog
	// from closing when it's double clicked.
	UINT nCommand = nID & 0xFFF0;
	if ( nCommand == SC_MOUSEMENU || nCommand == SC_DEFAULT )
	{
		// Toggle the pinned state.
		m_bPinned = !m_bPinned;
		UpdateIcon();
	}
    else
    {
        CDialog::OnSysCommand( nID, lParam );
    }
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
